﻿/**************************************************************************/
/*            角丸長方形に変換、または作成を行う（角丸は疑似）            */
/*                                                           by お～まち  */
/*                                              2017.12.30  Version 1.10  */
/**************************************************************************/
app.scriptPreferences.userInteractionLevel=UserInteractionLevels.interactWithAll;

if(app.documents.length == 0){
	alert("開いているドキュメントがありません。処理を中止します");
	exit();
}
var s = app.selection.length;
if(s > 0){
	for(i = 0; i < app.selection.length; i++){
		if(app.selection[i].constructor.name != "Rectangle"){
			alert("長方形でないオブジェクトが選択されています。処理を中止します");
			exit();
		}
	}
	for(i = 0; i < app.selection.length; i++){
		if(app.selection[i].rotationAngle != 0){
			alert("回転している長方形では実行できません。処理を中止します");
			exit();
		}
	}
	for(i = 0; i < app.selection.length; i++){
		if(app.selection[i].shearAngle != 0){
			alert("シアーのかかった長方形では実行できません。処理を中止します");
			exit();
		}
	}
}

const p = 0.4477; //ここが疑似角丸の所以
//使用する単位の設定
const un = ["ポイント", "インチ", "ミリメートル", "センチメートル", "歯", "アメリカ式ポイント"];
const ub = [2054188905, 2053729891, 2053991795, 2053336435, 1516790048, 1514238068];
const ux = [1, 0.0138889, 0.3527778, 0.03527778, 1.4112, 1.003921];
for(i = 0; i < ub.length; i++){
	if(app.activeDocument.viewPreferences.horizontalMeasurementUnits == ub[i]){
		var u = i;
		break;
	}
}
if(u == undefined) u = 2;
var umax = 15552 * ux[u]; //幅と高さの最大値（ドキュメントの最大値に合わせてあります）

//選択された長方形の幅と高さの最小値を取得
if(s > 0){
	var w = umax;
	var h = umax;
	for(i = 0; i < app.selection.length; i++){
		with(app.selection[i]){
			if(geometricBounds[2] - geometricBounds[0] < h) h = geometricBounds[2] - geometricBounds[0];
			if(geometricBounds[3] - geometricBounds[1] < w) w = geometricBounds[3] - geometricBounds[1];
		}
	}
	w = Math.round(w * 100000000) / 100000000;
	h = Math.round(h * 100000000) / 100000000;
}

//ここからダイアログを作成
dlg = app.dialogs.add();
if(s == 0) dlg.name = "角丸長方形の作成";
if(s >= 0) dlg.name = "角丸長方形へ変換";
dlc = dlg.dialogColumns.add();
with(dlc.dialogRows.add()){ //1行目
	if(s == 0){
		staticTexts.add({staticLabel:"長方形の幅："});
		dw = realEditboxes.add({minWidth:80, minimumValue:0, maximumValue:umax});
		staticTexts.add({staticLabel:"　長方形の高さ："});
		dh = realEditboxes.add({minWidth:80, minimumValue:0, maximumValue:umax});
	} else if(s == 1){
		staticTexts.add({staticLabel:"長方形の幅：" + w});
		staticTexts.add({staticLabel:"　長方形の高さ：" + h});
	} else {
		staticTexts.add({staticLabel:"幅の最小値：" + w});
		staticTexts.add({staticLabel:"　高さの最小値：" + h});
	}
}
with(dlc.dialogRows.add()){ //2行目
	staticTexts.add({staticLabel:"角丸の半径："});
	var dr = realEditboxes.add({minWidth:80, minimumValue:0});
	if(s == 0){
		dr.maximumValue = umax / 2;
		staticTexts.add({staticLabel:"　単位："});
		du = dropdowns.add({minWidth:80, stringList:un, selectedIndex:u});
	} else {
		if(w > h) dr.maximumValue = h / 2;
		if(w <= h) dr.maximumValue = w / 2;
		staticTexts.add({staticLabel:"　単位：" + un[u]});
	}
}
with(dlc.dialogRows.add()){ //3行目
	var de = enablingGroups.add({checkedState:false, staticLabel:"角丸の半径を個別に設定する"});
}
with(de.dialogColumns.add()){
	with(dialogRows.add()){
		staticTexts.add({staticLabel:"　左上："});
		dr0 = realEditboxes.add({minWidth:80, minimumValue:0, maximumValue:dr.maximumValue * 2});
	}
	with(dialogRows.add()){
		staticTexts.add({staticLabel:"　左下："});
		dr1 = realEditboxes.add({minWidth:80, minimumValue:0, maximumValue:dr.maximumValue * 2});
	}
}
with(de.dialogColumns.add()){
	with(dialogRows.add()){
		staticTexts.add({staticLabel:"　　右上："});
		dr3 = realEditboxes.add({minWidth:80, minimumValue:0, maximumValue:dr.maximumValue * 2});
	}
	with(dialogRows.add()){
		staticTexts.add({staticLabel:"　　右下："});
		dr2 = realEditboxes.add({minWidth:80, minimumValue:0, maximumValue:dr.maximumValue * 2});
	}
}
//半径の値のチェック
var err_flag = true;
while(err_flag){
	if(!dlg.show()){
		dlg.destroy();
		exit();
	}
	err_flag = false;
	if(s == 0){
		w = dw.editValue;
		h = dh.editValue;
		if(w == 0 || h == 0){
			alert("幅もしくは高さが入力されていないか、無効な値です。");
			err_flag = true;
		}
	}
	if(err_flag == false && de.checkedState == false){
		if(dr.editValue == 0){
			alert("角丸の半径が入力されていないか、無効な値です。");
			err_flag = true;
		} else if(dr.editValue > h / 2){
			alert("半径の大きさが幅の半分を超えています。");
			err_flag = true;
		} else if(dr.editValue > w / 2){
			alert("半径の大きさが高さの半分を超えています。");
			err_flag = true;
		}
	}
	if(err_flag == false && de.checkedState == true){
		if(dr0.editValue + dr1.editValue > h){
			alert("左上と左下の半径の合計が高さを超えています。");
			err_flag = true;
		}
		if(dr1.editValue + dr2.editValue > w){
			alert("左下と右下の半径の合計が幅を超えています。");
			err_flag = true;
		}
		if(dr2.editValue + dr3.editValue > h){
			alert("右上と右下の半径の合計が高さを超えています。");
			err_flag = true;
		}
		if(dr3.editValue + dr0.editValue > w){
			alert("左上と右上の半径の合計が高さを超えています。");
			err_flag = true;
		}
		if(dr0.editValue + dr1.editValue + dr2.editValue + dr3.editValue == 0){
			alert("丸くする角がありません。");
			err_flag = true;
		}
	}
}
var c = [0, 0, 0, 0];
if(de.checkedState == true){
	c[0] = dr0.editValue;
	c[1] = dr1.editValue;
	c[2] = dr2.editValue;
	c[3] = dr3.editValue;
} else {
	c[0] = c[1] = c[2] = c[3] = dr.editValue;
}
if(s == 0){
	u = du.selectedIndex;
	org_h = app.activeDocument.viewPreferences.horizontalMeasurementUnits;
	org_v = app.activeDocument.viewPreferences.verticalMeasurementUnits;
	app.activeDocument.viewPreferences.horizontalMeasurementUnits = ub[u];
	app.activeDocument.viewPreferences.verticalMeasurementUnits = ub[u];
	rec = app.activeDocument.rectangles.add();
	rec.geometricBounds = [0, 0, h, w];
	rec.select();
}
dlg.destroy();

for(i = 0; i < app.selection.length; i++){
	var tgt = app.selection[i].paths[0];
	t = tgt.parent.geometricBounds[0];
	l = tgt.parent.geometricBounds[1];
	b = tgt.parent.geometricBounds[2];
	r = tgt.parent.geometricBounds[3];
	for(j = 0; j < 4; j++) tgt.pathPoints.add();
	tgt.pathPoints[0].anchor = [l, t + c[0]];
	tgt.pathPoints[0].leftDirection = [l, t + c[0] * p];
	tgt.pathPoints[1].anchor = [l, b - c[1]];
	tgt.pathPoints[1].rightDirection = [l, b - c[1] * p];
	tgt.pathPoints[2].anchor = [l + c[1], b];
	tgt.pathPoints[2].leftDirection = [l + c[1] * p, b];
	tgt.pathPoints[3].anchor = [r - c[2], b];
	tgt.pathPoints[3].rightDirection = [r - c[2] * p, b];
	tgt.pathPoints[4].anchor = [r, b - c[2]];
	tgt.pathPoints[4].leftDirection = [r, b - c[2] * p];
	tgt.pathPoints[5].anchor = [r, t + c[3]];
	tgt.pathPoints[5].rightDirection = [r, t + c[3] * p];
	tgt.pathPoints[6].anchor = [r - c[3], t];
	tgt.pathPoints[6].leftDirection = [r - c[3] * p, t];
	tgt.pathPoints[7].anchor = [l + c[0], t];
	tgt.pathPoints[7].rightDirection = [l + c[0] * p, t];
}

if(s == 0){
	app.activeDocument.viewPreferences.horizontalMeasurementUnits = org_h;
	app.activeDocument.viewPreferences.verticalMeasurementUnits = org_v;
}
